import re

class SENTINAL():
	pass

def json_get_fast(json_string, key, default=SENTINAL()):
	'''
	uses regex to pull out the simple value of <key> in <json_string>, 
	without parsing the json object.  If <key> is not found
	return <default>, or if unspecified raise KeyError.

	Limitations:
		- <json_string> must be a plain json object e.g. '{"key":"val"...}'
		- the target value must be simple, e.g. null, number, or string
		- the first match for <key> (having a simple val) is returned 
			(it is ok if key arises within a nested object)

	Output:
		- all numbers are returned as floats. cast to int as desired.
		- null is returned as None
		- if the key is not found, KeyError is raised, unless <default>
			is specified, in which case <default> is returned
		- keys which have non-simple values (arrays or nested objects) are
			ignored (this can lead to KeyError if key is present but val
			is not simple).
	'''
	find_pattern = (
		r'"%s"'					# match the key
		r'\s*:\s*'				# match the colon
		r'(null|[\d\.]+|".*?")'	# match the val (null, number, or string)
	) % key
	find_re = re.compile(find_pattern)
	try:
		val_str = find_re.search(json_string).groups()[0]
	except (AttributeError, IndexError):
		if isinstance(default, SENTINAL):
			raise KeyError(key)
		else:
			return default

	if val_str == 'null':
		return None
	if val_str.startswith('"'):
		return val_str[1:-1]	# strip quotes
	return float(val_str)





def progress(i, total, period=1000, prefix=''):
	'''
	Prints a string representing the % completion when starting the
	`i`th step out of `total` steps, but only prints every `period` steps.
	'''
	if i % period == 0:
		print '%s%2.1f %%' % (prefix, i/float(total) * 100)



def invert_dict(dictionary):
	new_dict = {}
	for key in dictionary:
		new_dict[dictionary[key]] = key

	return new_dict


def merge_dicts(*dictionaries):
	merged = {}
	for dictionary in reversed(dictionaries):
		merged.update(dictionary)
	return merged


def dzip(*dictionaries):
	'''
	Like zip, but for dictionaries.  Produce a dictionary whose keys are 
	given by the intersection of input dictionaries' keys, and whose
	values are are tuples of the input dicts corresponding values.
	'''

	# Define the dzip of no dictionaries to be an empty dictionary
	if len(dictionaries) == 0:
		return {}

	# Get the keys common to all dictionaries
	keys = set(dictionaries[1])
	for d in dictionaries[1:]:
			keys &= set(d)

	# Make the zipped dictionary
	return {
		key : tuple([d.get(key, None) for d in dictionaries])
		for key in keys
	}




class Max(object):

	def __init__(self, keep_last=True):
		self.extreme_key = None
		self.extreme_item = None
		self.keep_last = keep_last

	def add(self, key, item):
		if (
			self.extreme_key is None 
			or self.comp(key, self.extreme_key)
			or (self.keep_last and key == self.extreme_key)
		):
			self.extreme_key = key
			self.extreme_item = item

	def update(self, key_item_pairs):
		for key, item in key_item_pairs:
			self.add(key, item)

	def get(self):
		return self.extreme_key, self.extreme_item

	def comp(self, key1, key2):
		return key1 > key2


class Min(object):
	def comp(self, key1, key2):
		return key1 < key2



from natsort import humansorted
import shutil
import re
import subprocess
import os


def file_empty(file_path):
	return os.stat(file_path).st_size == 0

def ensure_removed(path):
	"""
	Ensure that the given path does not exist, by removing it if necessary.
	"""
	if os.path.exists(path):
		shutil.rmtree(path)	

def ensure_exists(path):
	"""
	Ensure that the given path exists, by creating it if it doesn't.  If the
	path exists but is a file, an `OSError` would be raised.
	"""
	if not os.path.exists(path):
		os.makedirs(path)	


def ls(
	path,
	files=True,
	dirs=True,
	match=None,
	exclude=None,
	absolute=False,
	recurse=False,
	list_all=False,
	natural_sort=True,
	iteritems=False
):

	lister = PathLister(
		path=path,
		files=files,
		dirs=dirs,
		match=match,
		exclude=exclude,
		absolute=absolute,
		recurse=recurse,
		list_all=list_all,
		natural_sort=natural_sort,
		iteritems=iteritems
	)
	return lister.generate()


class PathLister(object):

	'''
		Iterator for listing all files in a directory.
		- Lists files found under path, unless <files> is False
		- Can also list directories, if <dirs> is True
		- Only files (and directories) matching <match> 
			will be returned, unless they also match <exclude>.
		- By default, doesn't descend into subdirectories, unless
			<recurse> is True, lists files in subdirs too.
		- If <recurse> is True, only directories matching <match>
			and not matching <exclude> will be followed
		- Paths to files are returned relative to <path>, unless <absolute>
			is true, then absolute paths are returned
		- Files are listed in natural sort ordering (i.e. respecting
			numerical order), but if <natural_sort> is False alphabetical
			order is used instead.
		- Hidden files (starting with '.') are ommitted, but if <list_all>
			is true, then they will be included.  In recursive mode, this
			is necessary in orderd to follow hidden  folders.
	'''

	def __init__(
		self,
		path,
		files=True,
		dirs=True,
		match=None,
		exclude=None,
		absolute=False,
		recurse=False,
		list_all=True,
		natural_sort=True,
		iteritems=False
	):
		self.path = path
		self.files = files
		self.dirs = dirs
		self.match = None if match is None else re.compile(match)
		self.exclude = None if exclude is None else re.compile(exclude)
		self.absolute = absolute
		self.recurse = recurse
		self.list_all = list_all
		self.natural_sort = natural_sort
		self.iteritems=iteritems


	def filter_back_pointers(self, dirs):
		'''
			Exclude '.' and '..' from <dirs>.
		'''
		return filter(
			lambda i: not (i.split('/')[-1]=='.' or i.split('/')[-1]=='..'),
			dirs
		)


	def generate(self):
		'''
		Either return the path generator, or a list of paths, depending on 
		the constructor.
		'''
		self.walker = os.walk(self.path)
		self.next_dir()
		if self.iteritems:
			return self._generate()
		else:
			return [item for item in self._generate()]


	def next_dir(self):
		cur_path, cur_dirs, cur_files = self.walker.next()

		# First reletavize cur_path to current working directory (by default
		# it's reletavized to the path given in the constructor)
		cur_path = os.path.relpath(cur_path, '.')

		# Do we want the files, the directories, both?
		if self.files:
			self.items = cur_files
		else:
			self.items = []
		if self.dirs:
			self.items.extend(cur_dirs)

		# Absolutize the paths if that's what was specified in the constructor
		if self.absolute:
			self.items = [
				os.path.abspath(os.path.join(cur_path, d)) for d in self.items
			]

		# Otherwise reletavize the files to the current working directory
		# But only necessary if cur_path isn't the current working directory
		elif cur_path != '.':
			self.items = [os.path.join(cur_path, f) for f in self.items]

		# Sort the items either "naturally" or alphabetically
		if self.natural_sort:
			self.items = humansorted(self.items, reverse=True)
		else:
			self.items.sort(reverse=True)

		# Filter items against the exclude and match if any
		if self.match is not None:
			self.items = [i for i in self.items if self.match.search(i)]
		if self.exclude is not None:
			self.items = [i for i in self.items if not self.exclude.search(i)]


	def _generate(self):
		'''
			yield the next item, and absolutize the path if necessary.
		'''
		# We'll break out of this loop when stopIteration is raised by 
		# the call to self.get_next()
		while True:
			yield self.get_next()


	def get_next(self):

		while True:

			# Try popping off the next item
			if self.files:
				try:
					return self.items.pop()
				except IndexError:
					pass

			# If there's no more items, and if we're in recursive mode, 
			# descend into the next directory
			if self.recurse:
				self.next_dir()
				return self.get_next()

			# And if we're not in recursive mode, then we're done
			else:
				raise StopIteration


	def _ls(self, path, filter_back_pointers=True):

		# Ask OS to list the files and directories in path

		#if self.list_all:
		#	items = check_output(['ls', '-a', path]).split()
		#else:
		#	items = check_output(['ls', path]).split()

		ls = subprocess.Popen(list_command, stdout=subprocess.PIPE)


		# Sort the files and directories
		if self.natural_sort:
			sort_command = ['sort', '-n', '-r']
		else:
			sort_command = ['sort', '-r']
		items = subprocess.check_output(sort_command, stdin=ls.stdout)
		ls.wait()
		items = [os.path.join(path, i) for i in items.split()]

		# Get rid of '.' and '..', if necessary
		if self.list_all and filter_back_pointers:
			items = self.filter_back_pointers(items)

		# Go through the list separate files from directories
		files = []
		dirs = []
		for item in items:

			# Skip if it doesn't match match, or does match exclude
			whitelist_match = (
				True if self.match is None 
				else self.match.search(item) 
			)
			blacklist_match = (
				false if self.exclude is None
				else self.exclude.search(item)
			)
			print blacklist_match
			if not whitelist_match or blacklist_match:
				continue

			# Append files to the list of files
			if os.path.isfile(item):
				files.append(item)

			# Append directories to the list of dirs
			else:
				dirs.append(item)

		return files, dirs




#!/usr/bin/env python
from setuptools import setup

setup(
	name='t4k',
	url='http://drgitlab.cs.mcgill.ca/enewe101/trix_for_kids',
	version='0.5.0',
	description='my personal swiss army knife',
	author='edward newell',
	author_email='edward.newell@gmail.com',
	packages=['t4k'],
	license='MIT',
	classifiers=[
		'Development Status :: 2 - Pre-Alpha',
		'Intended Audience :: Science/Research',
		'Intended Audience :: Information Technology',
		'Programming Language :: Python :: 2.7',
		'Natural Language :: English',
		'Topic :: Utilities',
	],
	keywords='trix are for kids',
	install_requires=['psutil', 'natsort']
)

